@@ -29,6 +29,7 @@
 
 #include <kwindowsystem.h>
 #include <klocalizedstring.h>
+#include <KAuthorized>
 
 #include <KPackage/Package>
 
@@ -223,6 +224,9 @@ void DesktopView::keyPressEvent(QKeyEvent *e)
         const QString text = e->text().trimmed();
         if (!text.isEmpty() && text[0].isPrint()) {
             const QString interface(QStringLiteral("org.kde.krunner"));
+            if (!KAuthorized::authorize(QStringLiteral("run_command"))) {
+                return;
+            }
             org::kde::krunner::App krunner(interface, QStringLiteral("/App"), QDBusConnection::sessionBus());
             krunner.query(text);
             e->accept();
@@ -266,6 +270,9 @@ void DesktopView::screenGeometryChanged()
     setGeometry(geo);
     setMinimumSize(geo.size());
     setMaximumSize(geo.size());
+    if (m_shellSurface) {
+        m_shellSurface->setPosition(geo.topLeft());
+    }
 }
 
 
@@ -292,5 +299,6 @@ void DesktopView::setupWaylandIntegration()
             return;
         }
         m_shellSurface = interface->createSurface(s, this);
+        m_shellSurface->setPosition(screen()->geometry().topLeft());
     }
 }
