@@ -37,8 +37,6 @@
 
 #include <Plasma/Containment>
 #include <Plasma/Package>
-#include <KScreen/Config>
-#include <KScreen/Output>
 
 #include <KWayland/Client/plasmashell.h>
 #include <KWayland/Client/surface.h>
@@ -55,6 +53,7 @@ PanelView::PanelView(ShellCorona *corona, QScreen *targetScreen, QWindow *parent
        m_offset(0),
        m_maxLength(0),
        m_minLength(0),
+       m_contentLength(0),
        m_distance(0),
        m_thickness(30),
        m_alignment(Qt::AlignLeft),
@@ -87,6 +86,7 @@ PanelView::PanelView(ShellCorona *corona, QScreen *targetScreen, QWindow *parent
     connect(&m_unhideTimer, &QTimer::timeout,
             this, &PanelView::restoreAutoHide);
 
+    m_lastScreen = targetScreen;
     connect(screen(), SIGNAL(geometryChanged(QRect)),
             &m_positionPaneltimer, SLOT(start()));
     connect(this, SIGNAL(locationChanged(Plasma::Types::Location)),
@@ -102,13 +102,9 @@ PanelView::PanelView(ShellCorona *corona, QScreen *targetScreen, QWindow *parent
     connect(&m_strutsTimer, &QTimer::timeout,
             this, &PanelView::updateStruts);
 
-    connect(m_corona->screensConfiguration()->screen().data(), &KScreen::Screen::currentSizeChanged,
-            this, &PanelView::updateStruts);
-
     qmlRegisterType<QScreen>();
     rootContext()->setContextProperty(QStringLiteral("panel"), this);
     setSource(QUrl::fromLocalFile(m_corona->kPackage().filePath("views", QStringLiteral("Panel.qml"))));
-    PanelShadows::self()->addWindow(this);
 }
 
 PanelView::~PanelView()
@@ -195,7 +191,6 @@ void PanelView::setOffset(int offset)
     positionPanel();
     emit offsetChanged();
     m_corona->requestApplicationConfigSync();
-    m_strutsTimer.start(STRUTSTIMERDELAY);
     emit m_corona->availableScreenRegionChanged();
 }
 
@@ -215,33 +210,23 @@ void PanelView::setThickness(int value)
 
     config().writeEntry("thickness", value);
     m_corona->requestApplicationConfigSync();
-    positionPanel();
+    resizePanel();
 }
 
 int PanelView::length() const
 {
-    int defaultLength = formFactor() == Plasma::Types::Vertical ? screen()->size().height() : screen()->size().width();
-    return config().isValid() ? config().readEntry<int>("length", defaultLength) : defaultLength;
+    return qMax(1, m_contentLength);
 }
 
 void PanelView::setLength(int value)
 {
-    if (value == length()) {
+    if (value == m_contentLength) {
         return;
     }
 
-    config().writeEntry("length", value);
-    m_corona->requestApplicationConfigSync();
-    positionPanel();
+    m_contentLength = value;
 
-
-    const int maxSize = screen()->size().width() - m_offset;
-    if (containment()->formFactor() == Plasma::Types::Vertical) {
-        resize(thickness(), qBound<int>(MINSIZE, value, maxSize));
-    //Horizontal
-    } else {
-        resize(qBound<int>(MINSIZE, value, maxSize), thickness());
-    }
+    resizePanel();
     emit m_corona->availableScreenRegionChanged();
 }
 
@@ -260,17 +245,12 @@ void PanelView::setMaximumLength(int length)
         setMinimumLength(length);
     }
 
-    if (formFactor() == Plasma::Types::Vertical) {
-        setMaximumHeight(length);
-    } else {
-        setMaximumWidth(length);
-    }
     config().writeEntry("maxLength", length);
     m_maxLength = length;
     emit maximumLengthChanged();
     m_corona->requestApplicationConfigSync();
 
-    positionPanel();
+    resizePanel();
 }
 
 int PanelView::minimumLength() const
@@ -288,17 +268,12 @@ void PanelView::setMinimumLength(int length)
         setMaximumLength(length);
     }
 
-    if (formFactor() == Plasma::Types::Vertical) {
-        setMinimumHeight(length);
-    } else {
-        setMinimumWidth(length);
-    }
     config().writeEntry("minLength", length);
     m_minLength = length;
     emit minimumLengthChanged();
     m_corona->requestApplicationConfigSync();
 
-    positionPanel();
+    resizePanel();
 }
 
 int PanelView::distance() const
@@ -317,6 +292,11 @@ void PanelView::setDistance(int dist)
     positionPanel();
 }
 
+Plasma::FrameSvg::EnabledBorders PanelView::enabledBorders() const
+{
+    return m_enabledBorders;
+}
+
 void PanelView::setVisibilityMode(PanelView::VisibilityMode mode)
 {
     if (m_visibilityMode == mode) {
@@ -376,23 +356,6 @@ void PanelView::positionPanel()
         slideLocation = KWindowEffects::BottomEdge;
         break;
     }
-    m_strutsTimer.stop();
-    m_strutsTimer.start(STRUTSTIMERDELAY);
-
-    if (formFactor() == Plasma::Types::Vertical) {
-        setMinimumSize(QSize(thickness(), m_minLength));
-        setMaximumSize(QSize(thickness(), m_maxLength));
-
-        emit thicknessChanged();
-        emit lengthChanged();
-    } else {
-        setMinimumSize(QSize(m_minLength, thickness()));
-        setMaximumSize(QSize(m_maxLength, thickness()));
-
-        emit thicknessChanged();
-        emit lengthChanged();
-    }
-
     const QPoint pos = geometryByDistance(m_distance).topLeft();
     setPosition(pos);
     if (m_shellSurface) {
@@ -400,38 +363,42 @@ void PanelView::positionPanel()
     }
 
     KWindowEffects::slideWindow(winId(), slideLocation, -1);
+
+    updateEnabledBorders();
 }
 
 QRect PanelView::geometryByDistance(int distance) const
 {
     QScreen *s = screen();
     QPoint position;
+    const QRect screenGeometry = s->geometry();
+
     switch (containment()->location()) {
     case Plasma::Types::TopEdge:
         switch (m_alignment) {
         case Qt::AlignCenter:
-            position = QPoint(QPoint(s->geometry().center().x(), s->geometry().top()) + QPoint(m_offset - size().width()/2, distance));
+            position = QPoint(QPoint(screenGeometry.center().x(), screenGeometry.top()) + QPoint(m_offset - width()/2, distance));
             break;
         case Qt::AlignRight:
-            position = QPoint(QPoint(s->geometry().x() + s->geometry().width(), s->geometry().y()) - QPoint(m_offset + size().width(), distance));
+            position = QPoint(QPoint(screenGeometry.x() + screenGeometry.width(), screenGeometry.y()) - QPoint(m_offset + width(), distance));
             break;
         case Qt::AlignLeft:
         default:
-            position = QPoint(s->geometry().topLeft() + QPoint(m_offset, distance));
+            position = QPoint(screenGeometry.topLeft() + QPoint(m_offset, distance));
         }
         break;
 
     case Plasma::Types::LeftEdge:
         switch (m_alignment) {
         case Qt::AlignCenter:
-            position = QPoint(QPoint(s->geometry().left(), s->geometry().center().y()) + QPoint(distance, m_offset - size().height()/2));
+            position = QPoint(QPoint(screenGeometry.left(), screenGeometry.center().y()) + QPoint(distance, m_offset - height()/2));
             break;
         case Qt::AlignRight:
-            position = QPoint(QPoint(s->geometry().left(), s->geometry().y() + s->geometry().height()) - QPoint(distance, m_offset + size().height()));
+            position = QPoint(QPoint(screenGeometry.left(), screenGeometry.y() + screenGeometry.height()) - QPoint(distance, m_offset + height()));
             break;
         case Qt::AlignLeft:
         default:
-            position = QPoint(s->geometry().topLeft() + QPoint(distance, m_offset));
+            position = QPoint(screenGeometry.topLeft() + QPoint(distance, m_offset));
         }
         break;
 
@@ -439,14 +406,14 @@ QRect PanelView::geometryByDistance(int distance) const
         switch (m_alignment) {
         case Qt::AlignCenter:
             // Never use rect.right(); for historical reasons it returns left() + width() - 1; see http://doc.qt.io/qt-5/qrect.html#right
-            position = QPoint(QPoint(s->geometry().x() + s->geometry().width(), s->geometry().center().y()) - QPoint(thickness() + distance, 0) + QPoint(0, m_offset - size().height()/2));
+            position = QPoint(QPoint(screenGeometry.x() + screenGeometry.width(), screenGeometry.center().y()) - QPoint(thickness() + distance, 0) + QPoint(0, m_offset - height()/2));
             break;
         case Qt::AlignRight:
-            position = QPoint(QPoint(s->geometry().x() + s->geometry().width(), s->geometry().y() + s->geometry().height()) - QPoint(thickness() + distance, 0) - QPoint(0, m_offset + size().height()));
+            position = QPoint(QPoint(screenGeometry.x() + screenGeometry.width(), screenGeometry.y() + screenGeometry.height()) - QPoint(thickness() + distance, 0) - QPoint(0, m_offset + height()));
             break;
         case Qt::AlignLeft:
         default:
-            position = QPoint(QPoint(s->geometry().x() + s->geometry().width(), s->geometry().y()) - QPoint(thickness() + distance, 0) + QPoint(0, m_offset));
+            position = QPoint(QPoint(screenGeometry.x() + screenGeometry.width(), screenGeometry.y()) - QPoint(thickness() + distance, 0) + QPoint(0, m_offset));
         }
         break;
 
@@ -454,21 +421,39 @@ QRect PanelView::geometryByDistance(int distance) const
     default:
         switch (m_alignment) {
         case Qt::AlignCenter:
-            position = QPoint(QPoint(s->geometry().center().x(), s->geometry().bottom() - thickness() - distance) + QPoint(m_offset - size().width()/2, 1));
+            position = QPoint(QPoint(screenGeometry.center().x(), screenGeometry.bottom() - thickness() - distance) + QPoint(m_offset - width()/2, 1));
             break;
         case Qt::AlignRight:
-            position = QPoint(s->geometry().bottomRight() - QPoint(0, thickness() + distance) - QPoint(m_offset + size().width(), -1));
+            position = QPoint(screenGeometry.bottomRight() - QPoint(0, thickness() + distance) - QPoint(m_offset + width(), -1));
             break;
         case Qt::AlignLeft:
         default:
-            position = QPoint(s->geometry().bottomLeft() - QPoint(0, thickness() + distance) + QPoint(m_offset, 1));
+            position = QPoint(screenGeometry.bottomLeft() - QPoint(0, thickness() + distance) + QPoint(m_offset, 1));
         }
     }
     QRect ret = formFactor() == Plasma::Types::Vertical ? QRect(position, QSize(thickness(), height())) : QRect(position, QSize(width(), thickness()));
-    ret = ret.intersected(s->geometry());
+    ret = ret.intersected(screenGeometry);
     return ret;
 }
 
+void PanelView::resizePanel()
+{
+    if (formFactor() == Plasma::Types::Vertical) {
+        const int minSize = qMax(MINSIZE, m_minLength);
+        const int maxSize = qMin(m_maxLength, screen()->size().height() - m_offset);
+        setMinimumSize(QSize(thickness(), minSize));
+        setMaximumSize(QSize(thickness(), maxSize));
+        resize(thickness(), qBound(minSize, m_contentLength, maxSize));
+    } else {
+        const int minSize = qMax(MINSIZE, m_minLength);
+        const int maxSize = qMin(m_maxLength, screen()->size().width() - m_offset);
+        setMinimumSize(QSize(minSize, thickness()));
+        setMaximumSize(QSize(maxSize, thickness()));
+        resize(qBound(minSize, m_contentLength, maxSize), thickness());
+    }
+    //positionPanel will be called implicitly from resizeEvent
+}
+
 void PanelView::restore()
 {
     if (!containment()) {
@@ -503,12 +488,6 @@ void PanelView::restore()
         const int maxSize = screen()->size().height() - m_offset;
         m_maxLength = qBound<int>(MINSIZE, m_maxLength, maxSize);
         m_minLength = qBound<int>(MINSIZE, m_minLength, maxSize);
-
-        resize(thickness(), qBound<int>(MINSIZE, length(), maxSize));
-
-        setMinimumHeight(m_minLength);
-        setMaximumHeight(m_maxLength);
-
     //Horizontal
     } else {
         defaultMaxLength = screen()->size().width();
@@ -520,14 +499,11 @@ void PanelView::restore()
         const int maxSize = screen()->size().width() - m_offset;
         m_maxLength = qBound<int>(MINSIZE, m_maxLength, maxSize);
         m_minLength = qBound<int>(MINSIZE, m_minLength, maxSize);
-
-        resize(qBound<int>(MINSIZE, length(), maxSize), thickness());
-
-        setMinimumWidth(m_minLength);
-        setMaximumWidth(m_maxLength);
     }
 
+
     setVisibilityMode((VisibilityMode)config().readEntry<int>("panelVisibility", (int)NormalPanel));
+    resizePanel();
 
     emit maximumLengthChanged();
     emit minimumLengthChanged();
@@ -590,77 +566,78 @@ void PanelView::restoreAutoHide()
 void PanelView::setAutoHideEnabled(bool enabled)
 {
 #if HAVE_X11
-    xcb_connection_t *c = QX11Info::connection();
-    if (!c) {
-        return;
+    if (QX11Info::isPlatformX11()) {
+        xcb_connection_t *c = QX11Info::connection();
+
+        const QByteArray effectName = QByteArrayLiteral("_KDE_NET_WM_SCREEN_EDGE_SHOW");
+        xcb_intern_atom_cookie_t atomCookie = xcb_intern_atom_unchecked(c, false, effectName.length(), effectName.constData());
+
+        QScopedPointer<xcb_intern_atom_reply_t, QScopedPointerPodDeleter> atom(xcb_intern_atom_reply(c, atomCookie, NULL));
+
+        if (!atom) {
+            return;
+        }
+
+        if (!enabled) {
+            xcb_delete_property(c, winId(), atom->atom);
+            return;
+        }
+
+        KWindowEffects::SlideFromLocation slideLocation = KWindowEffects::NoEdge;
+        uint32_t value = 0;
+
+        switch (location()) {
+        case Plasma::Types::TopEdge:
+            value = 0;
+            slideLocation = KWindowEffects::TopEdge;
+            break;
+        case Plasma::Types::RightEdge:
+            value = 1;
+            slideLocation = KWindowEffects::RightEdge;
+            break;
+        case Plasma::Types::BottomEdge:
+            value = 2;
+            slideLocation = KWindowEffects::BottomEdge;
+            break;
+        case Plasma::Types::LeftEdge:
+            value = 3;
+            slideLocation = KWindowEffects::LeftEdge;
+            break;
+        case Plasma::Types::Floating:
+        default:
+            value = 4;
+            break;
+        }
+
+        int hideType = 0;
+        if (m_visibilityMode == LetWindowsCover) {
+            hideType = 1;
+        }
+        value |= hideType << 8;
+
+        xcb_change_property(c, XCB_PROP_MODE_REPLACE, winId(), atom->atom, XCB_ATOM_CARDINAL, 32, 1, &value);
+        KWindowEffects::slideWindow(winId(), slideLocation, -1);
     }
-
-    const QByteArray effectName = QByteArrayLiteral("_KDE_NET_WM_SCREEN_EDGE_SHOW");
-    xcb_intern_atom_cookie_t atomCookie = xcb_intern_atom_unchecked(c, false, effectName.length(), effectName.constData());
-
-    QScopedPointer<xcb_intern_atom_reply_t, QScopedPointerPodDeleter> atom(xcb_intern_atom_reply(c, atomCookie, NULL));
-
-    if (!atom) {
-        return;
-    }
-
-    if (!enabled) {
-        xcb_delete_property(c, winId(), atom->atom);
-        return;
-    }
-
-    KWindowEffects::SlideFromLocation slideLocation = KWindowEffects::NoEdge;
-    uint32_t value = 0;
-
-    switch (location()) {
-    case Plasma::Types::TopEdge:
-        value = 0;
-        slideLocation = KWindowEffects::TopEdge;
-        break;
-    case Plasma::Types::RightEdge:
-        value = 1;
-        slideLocation = KWindowEffects::RightEdge;
-        break;
-    case Plasma::Types::BottomEdge:
-        value = 2;
-        slideLocation = KWindowEffects::BottomEdge;
-        break;
-    case Plasma::Types::LeftEdge:
-        value = 3;
-        slideLocation = KWindowEffects::LeftEdge;
-        break;
-    case Plasma::Types::Floating:
-    default:
-        value = 4;
-        break;
-    }
-
-    int hideType = 0;
-    if (m_visibilityMode == LetWindowsCover) {
-        hideType = 1;
-    }
-    value |= hideType << 8;
-
-    xcb_change_property(c, XCB_PROP_MODE_REPLACE, winId(), atom->atom, XCB_ATOM_CARDINAL, 32, 1, &value);
-    KWindowEffects::slideWindow(winId(), slideLocation, -1);
 #endif
 }
 
 void PanelView::resizeEvent(QResizeEvent *ev)
 {
     updateMask();
-    //don't setGeometry() to meke really sure we aren't doing a resize loop
+    //don't setGeometry() to make really sure we aren't doing a resize loop
     const QPoint pos = geometryByDistance(m_distance).topLeft();
     setPosition(pos);
     if (m_shellSurface) {
         m_shellSurface->setPosition(pos);
     }
+    m_strutsTimer.start(STRUTSTIMERDELAY);
     PlasmaQuick::ContainmentView::resizeEvent(ev);
 }
 
 void PanelView::moveEvent(QMoveEvent *ev)
 {
     updateMask();
+    m_strutsTimer.start(STRUTSTIMERDELAY);
     PlasmaQuick::ContainmentView::moveEvent(ev);
 }
 
@@ -680,21 +657,25 @@ void PanelView::integrateScreen()
 
 void PanelView::showEvent(QShowEvent *event)
 {
-    PanelShadows::self()->addWindow(this);
+    PanelShadows::self()->addWindow(this, enabledBorders());
     PlasmaQuick::ContainmentView::showEvent(event);
-    integrateScreen();
 
     //When the screen is set, the screen is recreated internally, so we need to
     //set anything that depends on the winId()
-    connect(this, &QWindow::screenChanged, this, [this](QScreen* screen) {
-        emit screenChangedProxy(screen);
+    connect(this, &QWindow::screenChanged, this, &PanelView::moveScreen, Qt::UniqueConnection);
+    integrateScreen();
+}
 
-        if (!screen)
-            return;
-        integrateScreen();
-        showTemporarily();
-        m_positionPaneltimer.start();
-    });
+void PanelView::moveScreen(QScreen* screen)
+{
+    emit screenChangedProxy(screen);
+    m_lastScreen = screen;
+
+    if (!screen)
+        return;
+    integrateScreen();
+    showTemporarily();
+    m_positionPaneltimer.start();
 }
 
 bool PanelView::event(QEvent *e)
@@ -856,37 +837,7 @@ void PanelView::updateMask()
             m_background->setImagePath(QStringLiteral("widgets/panel-background"));
         }
 
-        Plasma::FrameSvg::EnabledBorders borders = Plasma::FrameSvg::AllBorders;
-        switch (location()) {
-        case Plasma::Types::TopEdge:
-            borders &= ~Plasma::FrameSvg::TopBorder;
-            break;
-        case Plasma::Types::LeftEdge:
-            borders &= ~Plasma::FrameSvg::LeftBorder;
-            break;
-        case Plasma::Types::RightEdge:
-            borders &= ~Plasma::FrameSvg::RightBorder;
-            break;
-        case Plasma::Types::BottomEdge:
-            borders &= ~Plasma::FrameSvg::BottomBorder;
-            break;
-        default:
-            break;
-        }
-
-        if (x() <= screen()->geometry().x()) {
-            borders &= ~Plasma::FrameSvg::LeftBorder;
-        }
-        if (x() + width() >= screen()->geometry().x() + screen()->geometry().width()) {
-            borders &= ~Plasma::FrameSvg::RightBorder;
-        }
-        if (y() <= screen()->geometry().y()) {
-            borders &= ~Plasma::FrameSvg::TopBorder;
-        }
-        if (y() + height() >= screen()->geometry().y() + screen()->geometry().height()) {
-            borders &= ~Plasma::FrameSvg::BottomBorder;
-        }
-        m_background->setEnabledBorders(borders);
+        m_background->setEnabledBorders(enabledBorders());
 
         m_background->resizeFrame(size());
         setMask(m_background->mask());
@@ -899,12 +850,13 @@ void PanelView::updateStruts()
         return;
     }
 
+
     NETExtendedStrut strut;
 
     if (m_visibilityMode == NormalPanel) {
         const QRect thisScreen = screen()->geometry();
         // QScreen::virtualGeometry() is very unreliable (Qt 5.5)
-        const QRect wholeScreen = QRect(QPoint(0, 0), m_corona->screensConfiguration()->screen()->currentSize());
+        const QRect wholeScreen = QRect(QPoint(0, 0), screen()->virtualSize());
 
         //Extended struts against a screen edge near to another screen are really harmful, so windows maximized under the panel is a lesser pain
         //TODO: force "windows can cover" in those cases?
@@ -1097,5 +1049,48 @@ bool PanelView::edgeActivated() const
     return m_visibilityMode == PanelView::AutoHide || m_visibilityMode == LetWindowsCover;
 }
 
+void PanelView::updateEnabledBorders()
+{
+    Plasma::FrameSvg::EnabledBorders borders = Plasma::FrameSvg::AllBorders;
+
+    switch (location()) {
+    case Plasma::Types::TopEdge:
+        borders &= ~Plasma::FrameSvg::TopBorder;
+        break;
+    case Plasma::Types::LeftEdge:
+        borders &= ~Plasma::FrameSvg::LeftBorder;
+        break;
+    case Plasma::Types::RightEdge:
+        borders &= ~Plasma::FrameSvg::RightBorder;
+        break;
+    case Plasma::Types::BottomEdge:
+        borders &= ~Plasma::FrameSvg::BottomBorder;
+        break;
+    default:
+        break;
+    }
+
+    if (x() <= screen()->geometry().x()) {
+        borders &= ~Plasma::FrameSvg::LeftBorder;
+    }
+    if (x() + width() >= screen()->geometry().x() + screen()->geometry().width()) {
+        borders &= ~Plasma::FrameSvg::RightBorder;
+    }
+    if (y() <= screen()->geometry().y()) {
+        borders &= ~Plasma::FrameSvg::TopBorder;
+    }
+    if (y() + height() >= screen()->geometry().y() + screen()->geometry().height()) {
+        borders &= ~Plasma::FrameSvg::BottomBorder;
+    }
+
+    if (m_enabledBorders != borders) {
+
+        PanelShadows::self()->setEnabledBorders(this, borders);
+
+        m_enabledBorders = borders;
+        emit enabledBordersChanged();
+    }
+}
+
 
 #include "moc_panelview.cpp"
