@@ -27,11 +27,11 @@
 
 #include <QSet>
 #include <QTimer>
+#include <QDBusVariant>
+#include <QDBusContext>
 
 #include <KPackage/Package>
 
-#include <KScreen/Types>
-
 class Activity;
 class DesktopView;
 class PanelView;
@@ -66,7 +66,7 @@ namespace KWayland
     }
 }
 
-class ShellCorona : public Plasma::Corona
+class ShellCorona : public Plasma::Corona, QDBusContext
 {
     Q_OBJECT
     Q_PROPERTY(QString shell READ shell WRITE setShell)
@@ -99,8 +99,6 @@ public:
 
     Plasma::Containment *setContainmentTypeForScreen(int screen, const QString &plugin);
 
-    KScreen::ConfigPtr screensConfiguration() const;
-
     QScreen *screenForId(int screenId) const;
     void remove(DesktopView *desktopView);
 
@@ -133,9 +131,13 @@ public Q_SLOTS:
     ///DBUS methods
     void toggleDashboard();
     void setDashboardShown(bool show);
+    void loadInteractiveConsole();
     void showInteractiveConsole();
     void loadScriptInInteractiveConsole(const QString &script);
+    void showInteractiveKWinConsole();
+    void loadKWinScriptInInteractiveConsole(const QString &script);
     void toggleActivityManager();
+    void evaluateScript(const QString &string);
 
     Plasma::Containment *addPanel(const QString &plugin);
 
@@ -182,8 +184,7 @@ private Q_SLOTS:
     void addPanel(QAction *action);
     void populateAddPanelsMenu();
 
-    void outputEnabledChanged();
-    void addOutput(const KScreen::OutputPtr &output);
+    void addOutput(QScreen* screen);
     void primaryOutputChanged();
 
     void activityOpened();
@@ -197,11 +198,10 @@ private Q_SLOTS:
     void screenRemoved(QScreen* screen);
 
 private:
-    QScreen *outputToScreen(const KScreen::OutputPtr &output) const;
-    KScreen::OutputPtr screenToOutput(QScreen *screen) const;
+    void updateStruts();
     QScreen *insertScreen(QScreen *screen, int idx);
     void removeView(int idx);
-    bool isOutputRedundant(const KScreen::OutputPtr &output) const;
+    bool isOutputRedundant(QScreen* screen) const;
     void reconsiderOutputs();
     QList<PanelView *> panelsForScreen(QScreen *screen) const;
     DesktopView* desktopForScreen(QScreen *screen) const;
@@ -226,11 +226,10 @@ private:
     QAction *m_addPanelAction;
     QMenu *m_addPanelsMenu;
     KPackage::Package m_lookAndFeelPackage;
-    QSet<KScreen::OutputPtr> m_redundantOutputs;
+    QSet<QScreen*> m_redundantOutputs;
     QList<KDeclarative::QmlObject *> m_alternativesObjects;
     KDeclarative::QmlObject *m_interactiveConsole;
 
-    KScreen::ConfigPtr m_screenConfiguration;
     QTimer m_waitingPanelsTimer;
     QTimer m_appConfigSyncTimer;
     QTimer m_reconsiderOutputsTimer;
